local controls = {
    ["All"] = {
        ["texture"] = {
            ["up"]   = "/esoui/art/inventory/inventory_tabicon_all_up.dds",
            ["down"] = "/esoui/art/inventory/inventory_tabicon_all_down.dds",
            ["over"] = "/esoui/art/inventory/inventory_tabicon_all_over.dds",
            }},
    ["Stolen"] = {
        ["texture"] = {
            ["up"]   = "/esoui/art/vendor/vendor_tabIcon_fence_up.dds",
            ["down"] = "/esoui/art/vendor/vendor_tabIcon_fence_down.dds",
            ["over"] = "/esoui/art/vendor/vendor_tabIcon_fence_over.dds",
            }},
    ["Intricate"] = {
        ["texture"] = {
            ["up"]   = "/esoui/art/crafting/smithing_tabicon_research_up.dds",
            ["down"] = "/esoui/art/crafting/smithing_tabicon_research_down.dds",
            ["over"] = "/esoui/art/crafting/smithing_tabicon_research_over.dds",
            }},
    ["Trash"] = {
        ["texture"] = {
            ["up"]   = "/esoui/art/inventory/inventory_tabicon_junk_up.dds",
            ["down"] = "/esoui/art/inventory/inventory_tabicon_junk_down.dds",
            ["over"] = "/esoui/art/inventory/inventory_tabicon_junk_over.dds",
            }},
    }
local mode = "All"

local function DeconstructionFilterFunction(self, bagId, slotIndex, ...)
    if mode == "Stolen" then
        return not IsItemStolen(bagId, slotIndex)
    end

    if mode == "Intricate" then
        if GetItemTrait(bagId, slotIndex) == ITEM_TRAIT_TYPE_ARMOR_INTRICATE
        or GetItemTrait(bagId, slotIndex) == ITEM_TRAIT_TYPE_WEAPON_INTRICATE then
            return false
        end

        if FCOIsMarked then
            if FCOIsMarked and FCOIsMarked(GetItemInstanceId(bagId, slotIndex), 12) then return false end
            if FCOIsMarked(GetItemInstanceId(bagId, slotIndex), {1,2,3,4,5,6,7,8,9,10,11}) then return true end
        end
        if ItemSaver_IsItemSaved and ItemSaver_IsItemSaved(bagId, slotIndex) then return true end
        if IsItemStoredForCraftStore and IsItemStoredForCraftStore(Id64ToString(GetItemUniqueId(bagId, slotIndex))) then return true end

        if GetItemTrait(bagId, slotIndex) == ITEM_TRAIT_TYPE_ARMOR_NIRNHONED
        or GetItemTrait(bagId, slotIndex) == ITEM_TRAIT_TYPE_WEAPON_NIRNHONED then
            return true
        end
        
        if IsItemLinkCrafted(GetItemLink(bagId, slotIndex)) then 
            local hasSet = GetItemLinkSetInfo(GetItemLink(bagId, slotIndex), false)
            return hasSet
        end
        
        return true
    end

    if mode == "Trash" then
        if GetItemRequiredChampionPoints(bagId, slotIndex) > 140 then
            local craftingType = CRAFTING_TYPE_INVALID
            local itemType = GetItemType(bagId, slotIndex)

            if itemType == ITEMTYPE_ARMOR then
                local armorType = GetItemArmorType(bagId, slotIndex)
                if armorType == ARMORTYPE_MEDIUM
                or armorType == ARMORTYPE_LIGHT then
                    craftingType = CRAFTING_TYPE_CLOTHIER
                elseif armorType == ARMORTYPE_HEAVY then
                    craftingType = CRAFTING_TYPE_BLACKSMITHING
                end
            elseif itemType == ITEMTYPE_WEAPON then
                local weaponType = GetItemWeaponType(bagId, slotIndex)
                if weaponType == WEAPONTYPE_AXE
                or weaponType == WEAPONTYPE_DAGGER
                or weaponType == WEAPONTYPE_HAMMER
                or weaponType == WEAPONTYPE_SWORD
                or weaponType == WEAPONTYPE_TWO_HANDED_AXE
                or weaponType == WEAPONTYPE_TWO_HANDED_HAMMER
                or weaponType == WEAPONTYPE_TWO_HANDED_SWORD then
                    craftingType = CRAFTING_TYPE_BLACKSMITHING
                elseif weaponType == WEAPONTYPE_BOW
                or weaponType == WEAPONTYPE_FIRE_STAFF
                or weaponType == WEAPONTYPE_FROST_STAFF
                or weaponType == WEAPONTYPE_HEALING_STAFF
                or weaponType == WEAPONTYPE_LIGHTNING_STAFF
                or weaponType == WEAPONTYPE_SHIELD then
                    craftingType = CRAFTING_TYPE_WOODWORKING
                end
            end

            if craftingType ~= CRAFTING_TYPE_INVALID then
                local skillType, skillIndex = GetCraftingSkillLineIndices(craftingType)
                local abilityIndex = 1
                local currentLevel, maxLevel = GetSkillAbilityUpgradeInfo(skillType, skillIndex, abilityIndex)
                if currentLevel ~= maxLevel then return true end
            end
        end

        if FCOIsMarked then
            if FCOIsMarked and FCOIsMarked(GetItemInstanceId(bagId, slotIndex), 9) then return false end
            if FCOIsMarked(GetItemInstanceId(bagId, slotIndex), {1,2,3,4,5,6,7,8,10,11,12}) then return true end
        end
        if ItemSaver_IsItemSaved and ItemSaver_IsItemSaved(bagId, slotIndex) then return true end
        if IsItemStoredForCraftStore and IsItemStoredForCraftStore(Id64ToString(GetItemUniqueId(bagId, slotIndex))) then return true end

        if IsItemJunk(bagId, slotIndex) then
            return false
        end

        if GetItemTrait(bagId, slotIndex) == ITEM_TRAIT_TYPE_ARMOR_NIRNHONED
        or GetItemTrait(bagId, slotIndex) == ITEM_TRAIT_TYPE_ARMOR_ORNATE
        or GetItemTrait(bagId, slotIndex) == ITEM_TRAIT_TYPE_ARMOR_INTRICATE
        or GetItemTrait(bagId, slotIndex) == ITEM_TRAIT_TYPE_WEAPON_NIRNHONED
        or GetItemTrait(bagId, slotIndex) == ITEM_TRAIT_TYPE_WEAPON_ORNATE
        or GetItemTrait(bagId, slotIndex) == ITEM_TRAIT_TYPE_WEAPON_INTRICATE then
            return true
        end
        
	    if string.sub(GetItemName(bagId, slotIndex), 1, 12) == "The Master's"
        or string.sub(GetItemName(bagId, slotIndex), 1, 15) == "The Maelstrom's" then
            return true
        end

        if IsItemLinkCrafted(GetItemLink(bagId, slotIndex)) then return true end        
		
        local hasSet = GetItemLinkSetInfo(GetItemLink(bagId, slotIndex), false)
        return hasSet
    end
	
    return false
end

local function createButton(name, position)
    if WINDOW_MANAGER:GetControlByName("DeconstructionFilter" .. name, "") then return false end

    local button = WINDOW_MANAGER:CreateControl("DeconstructionFilter" .. name, ZO_SmithingTopLevelDeconstructionPanelInventory, CT_BUTTON)
    if button then
        button.name = name
        button:SetNormalTexture(controls[name].texture.up) 
        button:SetPressedTexture(controls[name].texture.down) 
        button:SetMouseOverTexture(controls[name].texture.over)
        button:SetState(mode == name and BSTATE_PRESSED or BSTATE_NORMAL)
        button:SetClickSound(SOUNDS.MENU_BAR_CLICK)
        button:SetDimensions(56, 56)
        button:SetAnchor(TOPLEFT, CRAFTSTATION_SMITHING_DECONSTRUCTION_TABS, TOPLEFT, 48 + 56 * position, 2)
        button:SetHandler("OnClicked", function(self)
            mode = self.name
            for name in pairs(controls) do
                WINDOW_MANAGER:GetControlByName("DeconstructionFilter" .. name, ""):SetState(mode == name and BSTATE_PRESSED or BSTATE_NORMAL)
            end
            SMITHING.deconstructionPanel.inventory:HandleDirtyEvent()
            end)
    end
    return false
end

ZO_PreHook(SMITHING.deconstructionPanel.inventory, "AddItemData", DeconstructionFilterFunction)
ZO_PreHook(ZO_Smithing, "SetMode", function(smithing_obj, mode)
    if mode == SMITHING_MODE_DECONSTRUCTION then
        createButton("All", 0)
        createButton("Stolen", 1)
        createButton("Intricate", 2)
        createButton("Trash", 3)
    end
    return false
end)
